@@ -2,6 +2,13 @@
 Eliot: Logging for Complex & Distributed Systems.
 """
 from warnings import warn
+from sys import version_info
+
+# Enable asyncio contextvars support in Python 3.5/3.6:
+if version_info < (3, 7):
+    import aiocontextvars
+    dir(aiocontextvars)  # pacify pyflakes
+    del aiocontextvars
 
 # Expose the public API:
 from ._message import Message
@@ -20,7 +27,6 @@ from ._validation import Field, fields, MessageType, ActionType, ValidationError
 from ._traceback import write_traceback, writeFailure
 from ._errors import register_exception_extractor
 from ._version import get_versions
-from ._asyncio import use_asyncio_context
 
 # Backwards compatibility:
 def add_destination(destination):
@@ -32,6 +38,10 @@ def add_destination(destination):
     )
     Logger._destinations.add(destination)
 
+# Backwards compatibility:
+def use_asyncio_context():
+    warn("This function is no longer as needed as of Eliot 1.8.0.",
+         DeprecationWarning, stacklevel=2)
 
 # Backwards compatibilty:
 addDestination = add_destination
