@@ -5,6 +5,7 @@ Tests for L{eliot._action}.
 from __future__ import unicode_literals
 
 import pickle
+import time
 from unittest import TestCase, skipIf
 from threading import Thread
 from warnings import catch_warnings, simplefilter
@@ -27,11 +28,11 @@ import testtools
 from testtools.matchers import MatchesStructure
 
 from .._action import (
-    Action, _ExecutionContext, current_action, startTask, start_action,
+    Action, current_action, startTask, start_action,
     ACTION_STATUS_FIELD, ACTION_TYPE_FIELD, FAILED_STATUS, STARTED_STATUS,
     SUCCEEDED_STATUS, DuplicateChild, InvalidStartMessage, InvalidStatus,
     TaskLevel, WrittenAction, WrongActionType, WrongTask, WrongTaskLevel,
-    TooManyCalls, log_call, _context_owner
+    TooManyCalls, log_call
 )
 from .._message import (
     EXCEPTION_FIELD,
@@ -67,187 +68,6 @@ from .strategies import (
 )
 
 
-class ExecutionContextTests(TestCase):
-    """
-    Tests for L{_ExecutionContext}.
-    """
-
-    def test_nothingPushed(self):
-        """
-        If no action has been pushed, L{_ExecutionContext.current} returns
-        C{None}.
-        """
-        ctx = _ExecutionContext()
-        self.assertIs(ctx.current(), None)
-
-    def test_pushSingle(self):
-        """
-        L{_ExecutionContext.current} returns the action passed to
-        L{_ExecutionContext.push} (assuming no pops).
-        """
-        ctx = _ExecutionContext()
-        a = object()
-        ctx.push(a)
-        self.assertIs(ctx.current(), a)
-
-    def test_pushMultiple(self):
-        """
-        L{_ExecutionContext.current} returns the action passed to the last
-        call to L{_ExecutionContext.push} (assuming no pops).
-        """
-        ctx = _ExecutionContext()
-        a = object()
-        b = object()
-        ctx.push(a)
-        ctx.push(b)
-        self.assertIs(ctx.current(), b)
-
-    def test_multipleCurrent(self):
-        """
-        Multiple calls to L{_ExecutionContext.current} return the same result.
-        """
-        ctx = _ExecutionContext()
-        a = object()
-        ctx.push(a)
-        ctx.current()
-        self.assertIs(ctx.current(), a)
-
-    def test_popSingle(self):
-        """
-        L{_ExecutionContext.pop} cancels a L{_ExecutionContext.push}, leading
-        to an empty context.
-        """
-        ctx = _ExecutionContext()
-        a = object()
-        ctx.push(a)
-        ctx.pop()
-        self.assertIs(ctx.current(), None)
-
-    def test_popMultiple(self):
-        """
-        L{_ExecutionContext.pop} cancels the last L{_ExecutionContext.push},
-        resulting in current context being whatever was pushed before that.
-        """
-        ctx = _ExecutionContext()
-        a = object()
-        b = object()
-        ctx.push(a)
-        ctx.push(b)
-        ctx.pop()
-        self.assertIs(ctx.current(), a)
-
-    def test_threadStart(self):
-        """
-        Each thread starts with an empty execution context.
-        """
-        ctx = _ExecutionContext()
-        first = object()
-        ctx.push(first)
-
-        valuesInThread = []
-
-        def inthread():
-            valuesInThread.append(ctx.current())
-
-        thread = Thread(target=inthread)
-        thread.start()
-        thread.join()
-        self.assertEqual(valuesInThread, [None])
-
-    def test_threadSafety(self):
-        """
-        Each thread gets its own execution context.
-        """
-        ctx = _ExecutionContext()
-        first = object()
-        ctx.push(first)
-
-        second = object()
-        valuesInThread = []
-
-        def inthread():
-            ctx.push(second)
-            valuesInThread.append(ctx.current())
-
-        thread = Thread(target=inthread)
-        thread.start()
-        thread.join()
-        # Neither thread was affected by the other:
-        self.assertEqual(valuesInThread, [second])
-        self.assertIs(ctx.current(), first)
-
-
-class ContextOwnerTests(TestCase):
-    """Tests for the shared global context."""
-
-    def test_reset(self):
-        """
-        Resetting resets to a normal C{_ExecutionContext}.
-        """
-        _context_owner.context = object()
-        _context_owner.reset()
-        self.assertIsInstance(_context_owner.context, _ExecutionContext)
-
-    def test_current_action(self):
-        """
-        L{current_action} returns the current value of the current global
-        context.
-        """
-        self.addCleanup(_context_owner.reset)
-        _context_owner.context.push("A")
-        self.assertEqual(current_action(), "A")
-        _context_owner.context.push("B")
-        self.assertEqual(current_action(), "B")
-        _context_owner.reset()
-        self.assertEqual(current_action(), None)
-        _context_owner.context.push("C")
-        self.assertEqual(current_action(), "C")
-
-    def test_set(self):
-        """
-        It's possible to set a new C{_ExecutionContext} once.
-        """
-        self.addCleanup(_context_owner.reset)
-
-        class A(_ExecutionContext):
-            pass
-
-        _context_owner.set(A)
-        self.assertIsInstance(_context_owner.context, A)
-
-    def test_set_twice(self):
-        """
-        Setting the same new C{_ExecutionContext} twice is no-op.
-        """
-        self.addCleanup(_context_owner.reset)
-
-        class A(_ExecutionContext):
-            pass
-
-        _context_owner.set(A)
-        a = _context_owner.context
-        _context_owner.set(A)
-        # Same object as before:
-        self.assertIs(_context_owner.context, a)
-
-    def test_set_twice_different_class(self):
-        """
-        Setting twice with different C{_ExecutionContext} classes raises a
-        C{RuntimeError}.
-        """
-        self.addCleanup(_context_owner.reset)
-
-        class A(_ExecutionContext):
-            pass
-
-        class B(_ExecutionContext):
-            pass
-
-        _context_owner.set(A)
-        with self.assertRaises(RuntimeError):
-            _context_owner.set(B)
-
-
 class ActionTests(TestCase):
     """
     Tests for L{Action}.
@@ -368,6 +188,23 @@ class ActionTests(TestCase):
         action.run(lambda: result.append(current_action()))
         self.assertEqual(result, [action])
 
+    def test_per_thread_context(self):
+        """Different threads have different contexts."""
+        in_thread = []
+
+        def run_in_thread():
+            action = Action(None, "", TaskLevel(level=[]), "")
+            with action.context():
+                time.sleep(0.5)
+                in_thread.append(current_action())
+
+        thread = Thread(target=run_in_thread)
+        thread.start()
+        time.sleep(0.2)
+        self.assertEqual(current_action(), None)
+        thread.join()
+        self.assertIsInstance(in_thread[0], Action)
+
     def test_runContextUnsetOnReturn(self):
         """
         L{Action.run} unsets the action once the given function returns.
