@@ -16,7 +16,10 @@ from ..testing import (
     assertHasMessage,
     assertHasAction,
     validate_logging,
-    capture_logging, )
+    capture_logging,
+    swap_logger,
+    check_for_errors
+)
 from .._output import MemoryLogger
 from .._action import start_action
 from .._message import Message
@@ -130,7 +133,7 @@ class LoggedActionTests(TestCase):
         is not found.
         """
         logger = MemoryLogger()
-        with start_action(logger, "test"):
+        with start_action(logger, action_type="test"):
             pass
         self.assertRaises(
             ValueError, self.fromMessagesIndex, logger.messages[1:], 0)
@@ -141,10 +144,12 @@ class LoggedActionTests(TestCase):
         is not found.
         """
         logger = MemoryLogger()
-        with start_action(logger, "test"):
+        with start_action(logger, action_type="test"):
             pass
-        self.assertRaises(
-            ValueError, self.fromMessagesIndex, logger.messages[:1], 0)
+        with self.assertRaises(ValueError) as cm:
+            self.fromMessagesIndex(logger.messages[:1], 0)
+        self.assertEqual(cm.exception.args[0],
+                         "Missing end message of type test")
 
     def test_fromMessagesAddsChildMessages(self):
         """
@@ -486,9 +491,12 @@ class ValidateLoggingTestsMixin(object):
                     "message_type": "wrongmessage"}, MESSAGE._serializer)
 
         test = MyTest()
-        self.assertRaises(ValidationError, test.debug)
-        self.assertEqual(
-            list(test.logger.messages[0].keys()), ["message_type"])
+        with self.assertRaises(ValidationError) as context:
+            test.debug()
+        # Some reference to the reason:
+        self.assertIn("wrongmessage", str(context.exception))
+        # Some reference to which file caused the problem:
+        self.assertIn("test_testing.py", str(context.exception))
 
     def test_addCleanupTracebacks(self):
         """
@@ -698,30 +706,6 @@ class CaptureLoggingTests(ValidateLoggingTestsMixin, TestCase):
             (test.recorded, result.skipped, result.errors, result.failures),
             (False, [(test, "Do not run this test.")], [], []))
 
-    def test_unflushedTracebacksDontFailForSkip(self):
-        """
-        If the decorated test raises L{SkipTest} then the unflushed traceback
-        checking normally implied by L{validateLogging} is also skipped.
-        """
-
-        class MyTest(TestCase):
-            @validateLogging(lambda self, logger: None)
-            def runTest(self, logger):
-                try:
-                    1 / 0
-                except:
-                    write_traceback(logger)
-                raise SkipTest("Do not run this test.")
-
-        test = MyTest()
-        result = TestResult()
-        test.run(result)
-
-        # Verify that there was only a skip, no additional errors or failures
-        # reported.
-        self.assertEqual((1, [], []),
-                         (len(result.skipped), result.errors, result.failures))
-
 
 MESSAGE1 = MessageType(
     "message1", [Field.forTypes("x", [int], "A number")],
@@ -945,3 +929,61 @@ class PEP8Tests(TestCase):
         L{validate_logging} is the same as L{validateLogging}.
         """
         self.assertEqual(validate_logging, validateLogging)
+
+
+class LowLevelTestingHooks(TestCase):
+    """Tests for lower-level APIs for setting up MemoryLogger."""
+
+    @capture_logging(None)
+    def test_swap_logger(self, logger):
+        """C{swap_logger} swaps out the current logger."""
+        new_logger = MemoryLogger()
+        old_logger = swap_logger(new_logger)
+        Message.log(message_type="hello")
+
+        # We swapped out old logger for new:
+        self.assertIs(old_logger, logger)
+        self.assertEqual(new_logger.messages[0]["message_type"], "hello")
+
+        # Now restore old logger:
+        intermediate_logger = swap_logger(old_logger)
+        Message.log(message_type="goodbye")
+        self.assertIs(intermediate_logger, new_logger)
+        self.assertEqual(logger.messages[0]["message_type"], "goodbye")
+
+    def test_check_for_errors_unflushed_tracebacks(self):
+        """C{check_for_errors} raises on unflushed tracebacks."""
+        logger = MemoryLogger()
+
+        # No errors initially:
+        check_for_errors(logger)
+
+        try:
+            1 / 0
+        except ZeroDivisionError:
+            write_traceback(logger)
+        logger.flush_tracebacks(ZeroDivisionError)
+
+        # Flushed tracebacks don't count:
+        check_for_errors(logger)
+
+        # But unflushed tracebacks do:
+        try:
+            raise RuntimeError
+        except RuntimeError:
+            write_traceback(logger)
+        with self.assertRaises(UnflushedTracebacks):
+            check_for_errors(logger)
+
+    def test_check_for_errors_validation(self):
+        """C{check_for_errors} raises on validation errors."""
+        logger = MemoryLogger()
+        logger.write({"x": 1, "message_type": "mem"})
+
+        # No errors:
+        check_for_errors(logger)
+
+        # Now long something unserializable to JSON:
+        logger.write({"message_type": object()})
+        with self.assertRaises(TypeError):
+            check_for_errors(logger)
